package sk.novotnyr.aidee;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

public class ID3Reader {
	private static int TAG_LENGTH = 128;
	
	private static int TITLE_LENGTH = 30;
	
	private static int TITLE_OFFSET = 3;

	private static int ARTIST_LENGTH = 30;
	
	private static int ARTIST_OFFSET = 3 + TITLE_LENGTH;

	public Tag read(File file) throws IOException, FileNotFoundException {
		if(file == null || !file.isFile()) {
			throw new IllegalArgumentException("Specified file " + file + " is not a proper file.");
		}
		byte[] tagData = loadTagData(file);
		
		Tag tag = new Tag();
		tag.setTitle(parseTitle(tagData));
                tag.setArtist(parseArtist(tagData));
		return tag;
	}
	
	private byte[] loadTagData(File file) throws IOException {
		long size = file.length();
		long tagPosition = size - TAG_LENGTH;
		FileInputStream fileIn = new FileInputStream(file);
		byte[] tagData = new byte[TAG_LENGTH];		
		fileIn.skip(tagPosition);
		int actualTagDataLength = fileIn.read(tagData);
		if(actualTagDataLength != TAG_LENGTH) {
			throw new IllegalArgumentException("Insufficient data");
		}
		return tagData;
	}
	
	private String parseArtist(byte[] tagData) {
		return parse(tagData, ARTIST_OFFSET, ARTIST_LENGTH);
	}

	private String parseTitle(byte[] tagData) {
		return parse(tagData, TITLE_OFFSET, TITLE_LENGTH);
	}

	private String parse(byte[] tagData, int offset, int length) {
		byte[] newData = new byte[length];
		int i = 0;
		for (; i < length; i++) {
			byte b = tagData[i + offset];;
			if(b == 0) {
				break;
			}
			newData[i] = b;
		}
		return new String(newData, 0, i);
	}
}
